searchNodes=[{"doc":"Parameter is a library for dealing with complex datatypes by solving the following problems: Schema creation and validation Input data validation Deserialization Serialization Motivation Offer a similar Schema model from the library Ecto to deal with complex data schemas. The main use case is to parse response from external apis. Parameter provides a well structured schema model which tries it's best to parse the external data. Schema The first step for building a schema for your data is to create a schema definition. This can be achieved by using the Parameter.Schema macro. defmodule UserSchema do use Parameter.Schema param do param :first_name , :string , key : &quot;firstName&quot; , required : true param :last_name , :string , key : &quot;lastName&quot; , required : true , default : &quot;&quot; param :age , :integer has_one :main_address , AddressSchema , key : &quot;mainAddress&quot; , required : true has_many :addresses , AddressSchema end end And the AddressSchema : defmodule AddressSchema do use Parameter.Schema param do param :city , :string , required : true param :street , :string param :number , :integer end end Each field needs to define the type that will be parsed and the options (if any). The available types are: :string :atom :integer :float :boolean :map :array :date :time :datetime :naive_datetime module * \\* Any module that implements the Parameter.Field behaviour is eligible to be a field in the schema definition. The options available for the field definition are: key : This is the key on the external source that will be converted to the param definition. As an example, if you receive data from an external source that uses a snake case for mapping first_name , this flag should be key: &quot;firstName&quot; . If this parameter is not set it will default to the field name. default : default value of the field. required : defines if the field needs to be present when parsing the input. After the definition, the schema can be validated and parsed against external parameters using the Parameter.load/3 function. Data Deserialization This is a common requirement when you receive data from an external source and want to validate and deserialize this data to an Elixir definition. This can be achieved using Parameter.load/2 or Parameter.load/3 functions: iex&gt; params = %{ &quot;mainAddress&quot; =&gt; %{ &quot;city&quot; =&gt; &quot;New York&quot; } , &quot;addresses&quot; =&gt; [ %{ &quot;city&quot; =&gt; &quot;Rio de Janeiro&quot; } ] , &quot;age&quot; =&gt; &quot;32&quot; , &quot;firstName&quot; =&gt; &quot;John&quot; , &quot;lastName&quot; =&gt; &quot;Doe&quot; , &quot;ASdf&quot; =&gt; &quot;asdf&quot; } ...&gt; Parameter . load ( UserSchema , params ) { :ok , %{ addresses : [ %{ city : &quot;Rio de Janeiro&quot; ] , age : 32 , first_name : &quot;John&quot; , last_name : &quot;Doe&quot; , main_address : %{ city : &quot;New York&quot; } } } Adding invalid data should return validation errors: iex&gt; params = %{ &quot;mainAddress&quot; =&gt; %{ &quot;city&quot; =&gt; &quot;New York&quot; , &quot;number&quot; =&gt; &quot;123AB&quot; } , &quot;addresses&quot; =&gt; [ %{ &quot;city&quot; =&gt; &quot;New York&quot; , &quot;number&quot; =&gt; &quot;123AB&quot; } , %{ &quot;city&quot; =&gt; &quot;Rio de Janeiro&quot; , &quot;number&quot; =&gt; &quot;Not number&quot; } ] , &quot;age&quot; =&gt; &quot;AA&quot; , &quot;firstName&quot; =&gt; &quot;John&quot; , &quot;lastName&quot; =&gt; &quot;Doe&quot; } ...&gt; Parameter . load ( UserSchema , params ) { :error , %{ addresses : [ &quot;0&quot; : %{ number : &quot;invalid integer type&quot; } , &quot;1&quot; : %{ number : &quot;invalid integer type&quot; } ] , age : &quot;invalid integer type&quot; , main_address : %{ number : &quot;invalid integer type&quot; } } } Custom Types For implementing custom types create a module that implements the Parameter.Parametrizable behaviour. Check the following example on how Integer parameter was implemented: defmodule IntegerCustomType do @moduledoc &quot;&quot;&quot; Integer parameter type &quot;&quot;&quot; @behaviour Parameter.Parametrizable @impl true # `load/1` is evaluated when parsing the parameters, you can do validations here and transform the data def load ( value ) when is_integer ( value ) do { :ok , value } end def load ( value ) when is_binary ( value ) do case Integer . parse ( value ) do { integer , &quot;&quot; } -&gt; { :ok , integer } _error -&gt; error_tuple ( ) end end def load ( _value ) do error_tuple ( ) end @impl true # `validate/1` checks the schema during compile time. It verifies the default value if it&#39;s passed to the schema validating its type def validate ( value ) when is_integer ( value ) do :ok end def validate ( _value ) do error_tuple ( ) end defp error_tuple , do : { :error , &quot;invalid integer type&quot; } end Custom modules can be used in Parameter.Schema defmodule UserSchema do use Parameter.Schema param do param :age , IntegerCustomType , required : true end end Unknown fields Loading will ignore fields that does not have a matching key in the schema. This behaviour can be changed with the following options: :ignore (default): ignore unknown fields :error : return an error with the unknown fields Installation Add parameter to your list of dependencies in mix.exs : def deps do [ { :parameter , &quot;~&gt; 0.1.1&quot; } ] end Also for add this depedency inside .formatter.exs file: import_deps : [ :ecto , :phoenix , :parameter ] , Roadmap Make the errors be returned on a list instead of single string. Validator: Schema should have a validator option where it can send a function with validation that returns :ok | {:error, reason} . Validation errors will be returned when loading the structure. Dump: Opposite of load.","ref":"Parameter.html","title":"Parameter","type":"module"},{"doc":"","ref":"Parameter.html#load/3","title":"Parameter.load/3","type":"function"},{"doc":"This module define the structure of a Field inside a Parameter Schema.","ref":"Parameter.Field.html","title":"Parameter.Field","type":"module"},{"doc":"","ref":"Parameter.Field.html#load/2","title":"Parameter.Field.load/2","type":"function"},{"doc":"","ref":"Parameter.Field.html#new!/1","title":"Parameter.Field.new!/1","type":"function"},{"doc":"","ref":"Parameter.Field.html#new/1","title":"Parameter.Field.new/1","type":"function"},{"doc":"","ref":"Parameter.Field.html#t:t/0","title":"Parameter.Field.t/0","type":"type"},{"doc":"Behaviour for implementing new parameter types.","ref":"Parameter.Parametrizable.html","title":"Parameter.Parametrizable","type":"behaviour"},{"doc":"","ref":"Parameter.Parametrizable.html#c:load/1","title":"Parameter.Parametrizable.load/1","type":"callback"},{"doc":"","ref":"Parameter.Parametrizable.html#c:validate/1","title":"Parameter.Parametrizable.validate/1","type":"callback"},{"doc":"Type behavour for implementing new types for parameters.","ref":"Parameter.Types.html","title":"Parameter.Types","type":"module"},{"doc":"","ref":"Parameter.Types.html#base_types/0","title":"Parameter.Types.base_types/0","type":"function"},{"doc":"","ref":"Parameter.Types.html#composite_types/0","title":"Parameter.Types.composite_types/0","type":"function"},{"doc":"","ref":"Parameter.Types.html#load/2","title":"Parameter.Types.load/2","type":"function"},{"doc":"","ref":"Parameter.Types.html#validate!/2","title":"Parameter.Types.validate!/2","type":"function"},{"doc":"","ref":"Parameter.Types.html#validate/2","title":"Parameter.Types.validate/2","type":"function"},{"doc":"","ref":"Parameter.Types.html#t:base_types/0","title":"Parameter.Types.base_types/0","type":"type"},{"doc":"","ref":"Parameter.Types.html#t:composite_types/0","title":"Parameter.Types.composite_types/0","type":"type"},{"doc":"","ref":"Parameter.Types.html#t:t/0","title":"Parameter.Types.t/0","type":"type"},{"doc":"Atom parameter type","ref":"Parameter.Types.Atom.html","title":"Parameter.Types.Atom","type":"module"},{"doc":"Boolean parameter type","ref":"Parameter.Types.Boolean.html","title":"Parameter.Types.Boolean","type":"module"},{"doc":"Date parameter type","ref":"Parameter.Types.Date.html","title":"Parameter.Types.Date","type":"module"},{"doc":"DateTime parameter type","ref":"Parameter.Types.DateTime.html","title":"Parameter.Types.DateTime","type":"module"},{"doc":"Float parameter type","ref":"Parameter.Types.Float.html","title":"Parameter.Types.Float","type":"module"},{"doc":"Integer parameter type","ref":"Parameter.Types.Integer.html","title":"Parameter.Types.Integer","type":"module"},{"doc":"Map parameter type","ref":"Parameter.Types.Map.html","title":"Parameter.Types.Map","type":"module"},{"doc":"NaiveDateTime parameter type","ref":"Parameter.Types.NaiveDateTime.html","title":"Parameter.Types.NaiveDateTime","type":"module"},{"doc":"String parameter type","ref":"Parameter.Types.String.html","title":"Parameter.Types.String","type":"module"},{"doc":"Time parameter type","ref":"Parameter.Types.Time.html","title":"Parameter.Types.Time","type":"module"}]